local __FILE__=tostring(debugstack(1,2,0):match("(.*):1:")) -- MUST BE LINE 1
local _,_,_,toc=GetBuildInfo()
local pp=print
--[[
Name: ABH-PoiRow.lua
Revision: $Rev: 426 $
Author: Alar of Daggerspine
Email: alar@aspide.it
Website: http://www.curse.com
SVN: $HeadUrl:$
License: GPL v2.1
--]]
local me, ns = ...
--[===[@debug@
print("Loading",__FILE__," inside ",me)
--@end-debug@]===]
if (LibDebug) then LibDebug() end
local function debug(...) 
--[===[@debug@
print(...)
--@end-debug@]===]
end
local print=_G.print
local notify=_G.print
local error=_G.error
local function dump() end
local function debugEnable() end
if (LibStub("AlarLoader-3.0",true)) then
	local rc=LibStub("AlarLoader-3.0"):GetPrintFunctions(me)
	print=rc.print
	--[===[@debug@
	debug=rc.debug
	dump=rc.dump
	--@end-debug@]===]
	notify=rc.notify
	error=rc.error
	debugEnable=rc.debugEnable
else
	debug("Missing AlarLoader-3.0")
end
debugEnable(false)
local L=LibStub("AceLocale-3.0"):GetLocale(me,true)
--[[ Standard prologue end --]]

--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local dbgalar=false
--[===[@debug@
dbgalar=true
--@end-debug@]===]
local Type, Version = "ABHPoiRow", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end
local AWG=LibStub("AlarWidgets-3.0")
local InjectStandardMethods=AWG.InjectStandardMethods
local C=LibStub("AlarCrayon-3.0"):GetColorTable()
local abh=LibStub("AceAddon-3.0"):GetAddon(me)
local bs=LibStub("LibBattleScan")

-- Lua APIs
local max, select, pairs, floor = _G.math.max, _G.select, _G.pairs ,_G.math.floor

-- WoW APIs
local CreateFrame, UIParent = _G.CreateFrame, _G.UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

local poitexture="interface\\minimap\\poiicons"

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local function IndexToTexCoord(i,l)
   i=i or 5
   l=l or 14
   local c=i%l
   local dh=1/l
   local dv=0.5/l
   local r=i-c
   local r=r/l
   local a=floor(c/4)
   return  c*dh-a*0.005,c*dh+dh-a*0.005,r*dv,r*dv+dv
end  


--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {}
function methods:OnAcquire()
	-- height is set dynamically by the text and image size
	self:SetWidth(200)
	self:SetHeight(16)
	self:SetText()
	self:SetTimerText()
	--self:SetImage(nil)
	self:SetImageSize(16, 16)
end
function methods:OnRelease()
	self.changed=0
end

-- ["OnRelease"] = nil,
function methods:SetText(text)
	self.label.string:SetText(text)

end
function methods:SetFormattedText(...)
	self.label.string:SetFormattedText(...)
end
function methods:SetTimerText(text)
	self.timer.string:SetText(text)
end
function methods:SetTimerFormattedText(...)
	self.timer.string:SetFormattedText(...)
end

function methods:SetColor(r, g, b)
	if not (r and g and b) then
		r, g, b = 1, 1, 1
	end
	self.label.string:SetVertexColor(r, g, b)
end

function methods:SetTimerColor(r, g, b)
	if not (r and g and b) then
		r, g, b = 1, 1, 1
	end
	self.timer.string:SetVertexColor(r, g, b)
end
function methods:Reset()
	local icon=self.textureIndex
	self.changed=GetTime()
	self:Refresh()
end
function methods:Refresh()
	local elapsed=floor(self.countdown - (floor(GetTime()-self.changed)))
	self.name,self.description,self.textureIndex,self.x,self.y=GetMapLandmarkInfo(self.index)
	self:SetIcon(self.textureIndex)
	if dbgalar then
		self:SetFormattedText("%d.%s",self.index,tostring(self.name))
	else
		self:SetFormattedText("%s",tostring(self.name))
	end
	local inConflict,faction=bs:isInConflict(self.textureIndex)	
	if abh:IsInRange(self.x or 0,self.y or 0) then
		self:SetColor(C.Yellow())
	else
		if (inConflict) then
			if (faction=="a") then
				self:SetTimerColor(C.Alliance())
			elseif (faction=="h") then
				self:SetTimerColor(C.Horde())
			end
		else
			if (faction=="d") then -- destroyed
				self:SetColor(C.LightGrey())
			else
				self:SetColor(C.White())
			end
		end		
	end
	if (not inConflict) then
		self:SetTimerText("")
		return
	end
	if (self.elapsed ~= elapsed and inConflict) then
		self.elapsed=elapsed
		if (elapsed >=0) then
			if (elapsed==0) then
				self:SetTimerText('')
				self:SetTimerColor(C.White())
			else
				self:SetTimerFormattedText("%d:%02d",floor(elapsed/60),elapsed%60)
				if (elapsed<11) then
					if (elapsed % 2) == 0 then
						self:SetTimerColor(C.Yellow())
					else
						self:SetTimerColor(C.Red())
					end
				end
			end
		end
	end

end
function methods:SetImage(path, ...)
	local image = self.image
	image:SetTexture(path)
	
	if image:GetTexture() then
		self.imageshown = true
		local n = select("#", ...)
		if n == 4 or n == 8 then
			image:SetTexCoord(...)
		else
			image:SetTexCoord(0, 1, 0, 1)
		end
	else
		self.imageshown = nil
	end
end
function methods:SetIcon(i)
	local image = self.image
	image:SetTexCoord(IndexToTexCoord(i))
end

function methods:SetFont(font, height, flags)
	self.label:SetFont(font, height, flags)
	self.timer:SetFont(font, height, flags)
end

function methods:SetFontObject(font)
	self:SetFont((font or _G.GameFontHighlightSmall):GetFont())
end

function methods:SetImageSize( width, height)
	height=height or width
	self.image:SetWidth(width)
	self.image:SetHeight(height)
	self.frame:SetHeight(height)
	self.label:SetHeight(height)
	self.timer:SetHeight(height)
end
function methods:Resize(width)
	width=width-self.image:GetWidth()-self.timer:GetWidth()
	self.label:SetWidth(width)
	
end

function methods:SetTimerSize( width, height)
	height=height or self.image:GetHeight()
	self.timer:SetWidth(width)
	self.timer:SetHeight(height)
end
function methods:SetPoi(i,countdown)
	self.countdown=tonumber(countdown or 10)
	self.index=i
	self.changed=-1
	self.elapsed=-1
	self:Reset()
end
function methods:Show()
	self.frame:Show()
end
function methods:Hide()
	self.frame:Hide()
end
function methods:SetAlpha()
	self.frame:SetAlpha()
end

local function Control_OnEnter(frame)
	frame.obj:Fire("OnEnter")
end

local function Control_OnLeave(frame)
	frame.obj:Fire("OnLeave")
end

local function Label_OnClick(frame, button)
	frame.obj.parent:Fire("OnPoiClick", button,frame.type,frame.string:GetText(),frame.obj)
	AceGUI:ClearFocus()
end
function methods:ClickTimer(button)
	Label_OnClick(self.timer,button or "LeftButton")
end
function methods:ClickLabel(button)
	Label_OnClick(self.label,button or "LeftButton")
end


--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function mkLabel(frame,align,width)
	local label =  CreateFrame("Frame", nil, frame)
	label.changed=0
	label:EnableMouse(true)
	label.string=label:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	label.string:SetJustifyH(align)
	label.string:SetJustifyV("TOP")
	label.string:SetAllPoints()
    local htexture=label:CreateTexture(nil,"HIGHLIGHT")
    htexture:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
    htexture:SetVertexColor(1,0.8,0)
    htexture:SetBlendMode("ADD")
    htexture:SetAllPoints(label)	
    label:SetWidth(width)	
    label:SetHeight(16)	
    --label:SetScript("OnEnter", Control_OnEnter)
    --label:SetScript("OnLeave", Control_OnLeave)
    label:SetScript("OnMouseDown", Label_OnClick)
    return label
end


local function Constructor()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	
	local image = frame:CreateTexture(nil, "BACKGROUND")
	image:SetTexture(poitexture,IndexToTexCoord(5))
	local label=mkLabel(frame,"LEFT",200)
	local timer=mkLabel(frame,"LEFT",40)
	label.type="label"
	timer.type="timer"
	timer:ClearAllPoints()
	label:ClearAllPoints()
	image:SetPoint("TOPLEFT")
	timer:SetPoint("TOPRIGHT")
	label:SetPoint("TOPLEFT",image,"TOPRIGHT",0,0)
	label:SetPoint("TOPRIGHT",timer,"TOPLEFT",0,0)
	-- create widget
	local widget = {
		label = label,
		image = image,
		frame = frame,
		timer =timer,
		type  = Type
	}
	InjectStandardMethods(widget)		
	for method, func in pairs(methods) do
		widget[method] = func
	end
	label.obj=widget
	timer.obj=widget
	--widget.highlight = highlight	
	
	
	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
